function bSuccess = RandomAttack(m,n,Z,bShow)
% bSuccess = RandomAttack(m,n,Z,bShow)
%
% Perform a random atatck with m observed login sessions.
% The password length is n and the modulus is Z.
% If bShow = 1, display the intermediate information about the attack.
% Default values: m=2, n=4, Z=10, bShow=1.
% 
% Programmed by Shujun Li in 2009.

bSuccess = 0;
if nargin<4
    bShow = 1;
    if nargin<3
        Z = 10;
        if nargin<2
            n = 4;
            if nargin<1
                m = 2;
            end
        end
    end
end

X = randint(n,1,Z);
a = randint(1,1,Z);

if bShow
    fprintf('The original password: X=%s; a=%d\n', mat2str(X), a);
end

% Collect m login sessions.
[Ys,cs,Ks] = VPSs(X,a,m,Z);

% Crack the password and the secret integer.
[astar,Xstar] = VPScracker(Ys,Ks,Z,bShow);
if isempty(astar)
    return; % The observed login sessions are not enough for breaking the secret integer a!
end
if astar~=a % This should never happen. Reserved for debugging purpose.
    disp('The cracked value of a is wrong! Attack aborted!');
    return;
end

if bShow
    fprintf('The cracked password: X*=%s; a=%d\n', mat2str(Xstar), astar);
end

% Impersonate the user.
[Ystar,cstar,Kstar] = VPSstar(Xstar,astar,Z);
% The server authenticates the user.
bSuccess = VerifyVPS(X,a,Ystar,Kstar,Z);

if bShow
    if bSuccess
        disp('Welcome Alice! (The attacker succeeded!)');
    else
        disp('You are not Alice! (The attacker failed!)');
    end
end
