function K = VPScY(X,a,Y,c,Z)
% K = VPScY(X,a,Y,c,Z)
% 
% Given a fixed password X, a secret integer a, the random salt (challenge)
% Y and the random integer c, generate the dynamic password (response) K.
% Z is the modulus (default: 10).
% The output K is a column vector.
% 
% Programmed by Shujun Li in 2009.

K = [];
if nargin<4
    disp('At least four arguments are needed!');
    return;
end
if nargin<5
    Z = 10;
end

n = numel(X);
K = zeros(n,1);
if numel(Y)~=n
    disp('The random salt (challenge) should be of the same size as the password X!');
end

K(1) = mod(a*X(1) + X(2) + Y(1) + c, Z);
for i=2:n-1
    K(i) = mod(a*K(i-1) + X(i) + X(i+1) + Y(i) + c, Z);
end
K(n) = mod(a*K(n-1) + X(n) + X(1) + Y(n) + c, Z);
