function [astar,Xstar] = VPScracker(Ys,Ks,Z,bShow)
% [astar,Xstar] = VPScracker(Ys,Ks,Z)
% 
% Given a number of observed login sessions stored in (Ys,Ks), crack the
% secret integer a (stored in astar) and get an equivalent password Xstar.
% Z is the modulus (default: 10).
% 
% Programmed by Shujun Li in 2009.

astar = [];
Xstar = [];

if nargin<2
    disp('At least two arguments are needed!');
    return;
end
if nargin<4
    bShow = 1;
    if nargin<3
        Z = 10;
    end
end

Ys_size = size(Ys);
if ~isequal(Ys_size, size(Ks))
    disp('The first two arguments should be of the same size!');
    return;
end

n = Ys_size(1);
m = Ys_size(2);
if m<2
    disp('At least two observed login sessions are necessary to crack the password!');
    return;
end

% Find a modular invertible value (k_{i-1}'-k_{i-1}).
[s1,s2,i_1] = FindInvertibleDeltaK(Ks,n,m,Z);
if i_1==0
    if bShow
        disp('The observed login sessions are not enough for breaking the password!');
    end
    return;
end
deltaK = Ks(i_1,s1) - Ks(i_1,s2);
i = i_1 + 1;
% Calculate the secret integer a.
astar = invmod(deltaK,Z) * ...
        ((Ks(i,s1)-Ks(i,s2)) - (Ys(i,s1)-Ys(i,s2)) - ...
        (Ks(1,s1)-Ks(1,s2)) + (Ys(1,s1)-Ys(1,s2)));
astar = mod(astar, Z);
% Calculate the equivalent password X^*.
Xstar = zeros(n,1);
Xstar(1) = mod(Ks(1,s1)-Ys(1,s1), Z);
for i=2:n
    Xstar(i) = mod(Ks(i,s1)-astar*Ks(i-1,s1)-Ys(i,s1), Z);
end

% Look for the first modular invertible delta K.
function [s1,s2,i] = FindInvertibleDeltaK(Ks,n,m,Z)

for Ks1=1:m
    for Ks2=Ks1+1:m
        for ii=1:n-1 % The maximal value of ii is n-1, NOT n.
            if gcd(Ks(ii,Ks1)-Ks(ii,Ks2),Z)==1
                s1 = Ks1;
                s2 = Ks2;
                i = ii;
                return;
            end
        end
    end
end
% Fail to find any modular invertible \Delta k_i
s1 = 0;
s2 = 0;
i = 0;
