function [Ystar,cstar,Kstar] = VPSstar(Xstar,astar,Z)
% [Ystar,cstar,Kstar] = VPSstar(Xstar,astar,Z)
% 
% Given a fixed password Xstar and a secret integer astar, generate ONE
% challenge-response pair (Kstar,Ystar) with a randomly generated integer cstar.
% Z is the modulus (default: 10).
% The outputs Kstar and Ystar are both column vectors.
% 
% Programmed by Shujun Li in 2009.

Ystar = [];
cstar = [];
Kstar = [];
if nargin<2
    disp('At least two arguments are needed!');
    return;
end
if nargin<3
    Z = 10;
end

Xstar = Xstar(:); % Transform Xstar to astar column vector.
n = numel(Xstar);
Kstar = zeros(n,1);

% The server generates the random salt Ystar and the random integer cstar.
cstar = randint(1,1,Z);
Ystar = randint(n,1,Z);

Kstar(1) = mod(Xstar(1) + Ystar(1) + cstar, Z);
for i=2:n
    Kstar(i) = mod(astar*Kstar(i-1) + Xstar(i) + Ystar(i) + cstar, Z);
end
